$(function () {
    /**
     @version 7.1
     @date 2019-10-07
     @author abernh <a.bernhard@23go.de>

     ## USAGE
     This needs
     - a branch element on the page reacting on the trackfield value, default is "userfield1 eq 'leaving_survey'"
     - a html element that loads this script
     - a html element below that that contains the modal content and will be auto-hidden

     Drop the following script block into the first HTML element.

     ```
     <script src="modal_button.v7.leavesurvey.se.js"></script>
     <script id="modalButtonScript">
     $(function(){
            SePlay.Leavesurvey({..optional parameters}); // see defaultOptions for parameters
        })
     </script>

     ```

     Example for a "leave survey" modal:

     ```
     <script src="modal_button.v7.leavesurvey.se.js"></script>
     <script id="modalButtonScript">
     $(function(){
            SePlay.Leavesurvey({
                onlyConfirm: false,
                openModalButtonPosition: "topright"
                onConfirm: function(){
                    $('input[name=PLAYSID]')
                        .clone()
                        .attr('name', '_vs')
                        .appendTo('form');

                    $('#se_next_button').trigger('click');
                }
            }); // see defaultOptions for parameters
        })
     </script>

     ```


     The 2nd HTML element on that page will be hidden and on-button-click displayed in a popup.

     If there is a value set for the "trackField", the "trackValue" will written to the backend on opening the modal.
     On closing the modal, this field value is reset to be empty.



     ## CHANGELOG
     2019-10-07 v7.1
     - added generic option 'openModalButtonPosition'

     */

        // SETTINGS

    var
        LOG_LABEL = "SE_LEAVE",
        defaultOptions = {
            thisId: "modalButtonScript",

            debug: false,

            onlyConfirm: true,
            openModalButtonLabel: "Leave survey",
            openModalButtonPosition: "bottomcenter", // "inline", "topright", "bottomcenter" (default), DomNodeObject
            openModalButtonTopRight: false, // deprecated since v7.1

            buttonStyles: {
                'font-size': '120%'
            },

            trackField: "userfield1",
            trackValue: "leaving_survey",
            trackValueCancelled: "leaving_cancelled",

            onConfirm: function(){
                $('input[name=PLAYSID]')
                    .clone()
                    .attr('name', '_vs')
                    .appendTo('form');

                $('#se_next_button').trigger('click');
            }
        };


    // SETTINGS -- end -------------------------------------------------------------------------

    var
        Leavesurvey = function(){
            var _this = this;

            _this.v = {
                destroyQueue: [],
                isInit: false
            };

            return function(options){
                if(options === "destroy"){
                    _this.destroy.apply(_this, [options])
                }else{
                    _this.init.apply(_this, [options])
                }
            }
        },
        _proto = Leavesurvey.prototype;

    _proto.version = "7.1";

    _proto.init = function(options){

        options = options || {};
        for(var i in defaultOptions){
            if(typeof(options[i]) === "undefined"){
                options[i] = defaultOptions[i];
            }
        }

        this.hideNextItem(options.thisId);

        if(this.v.isInit){
            return;
        }
        this.v.isInit = true;

        this.options = options;
        this.createModalButton();
    };

    _proto.hideNextItem = function(thisId){
        var _this = this,
            $thisItem = $('#' + thisId).closest('.se_item'),
            $nextItem = $thisItem.next();

        // HIDING NEXT ITEM
        if(!$('#modalContentStyles').length)
            $('body').append('<style id="modalContentStyles">.hiddenLeavesurveyModalContent{ display: none !important; }</style>');

        $nextItem.css('display', 'none!important').addClass('hiddenLeavesurveyModalContent');
    };

    _proto.createModalButton = function(){
        var _this = this,
            options = this.options,
            $thisItem = $('#' + options.thisId).closest('.se_item'),
            $nextItem = $thisItem.next();

        // GET ELEMENTS
        var
            $footer = $('.se_footer'),
            $page = $('.se_item').parent(),
            $form = $('form[name=play]'),
            $openModalBtn = $('<button type="button" class="btn btn-default" id="buttonLeave">' + options.openModalButtonLabel +
                '</button>'),
            $modal = $('<div id="se_LeavesurveyModal">'+
                '<div style="position:fixed;top:0;left:0;right:0;bottom:0;background:rgba(0,0,0,0.5);z-index:999;">' +
                '<div style="display:flex;align-items:center;justify-content:center;height:100%;width:100%;">' +
                '<div style="width:400px;max-width:90%;padding:1em;border:1px solid gray;background:white;">' +
                '<div class="modal_body"></div>' +
                '<div class="modal_footer" style="margin-top:1em;border-top:1px solid gray;padding-top:1em;">' +
                '<button class="btn btn-default modal_button_close">back</button>' +
                '<button class="btn btn-primary modal_button_confirm" style="float:right;">' + options.openModalButtonLabel + '</button>' +
                '</div></div></div></div></div>'
            ),
            $modalBody = $('.modal_body', $modal),
            $modalBtnCancel = $('.modal_button_close', $modal),
            $modalBtnConfirm = $('.modal_button_confirm', $modal);

        function doTrackField(field, value) {
            if (field == undefined || value == undefined) return;
            var data = {
                _resp_op: 'next'
            };
            $form.serializeArray().map(function (field0) {
                if(["form_name","PLAYSID"].indexOf(field0.name) !== -1){
                    data[field0.name] = field0.value;
                }
            });
            data['r##' + field] = value;
            return $.post(location.href, data);
        }

        // SETUP MODAL
        $modal.hide();
        $modalBody.html($nextItem.find('.se_text > div').html());

        $('body #se_LeavesurveyModal').remove(); // remove previous left-over modals.
        $('body').append($modal);
        this.v.destroyQueue.push(function(){
            $modal.remove();
        });

        // SETUP BUTTONS
        $modalBtnCancel.css(options.buttonStyles);
        $modalBtnCancel.on('click', function () {
            $modal.hide();
            if(options.trackField && options.trackValueCancelled){
                doTrackField(options.trackField, options.trackValueCancelled);
            }
        });
        $modalBtnConfirm.css(options.buttonStyles);
        $modalBtnConfirm.on('click', function () {
            $(this).attr('disabled', true);

            if(options.trackField && options.trackValue){
                doTrackField(options.trackField, options.trackValue).then(function(){
                    options.onConfirm();
                    _this.destroy();
                });
            }else{
                options.onConfirm();
                _this.destroy();
            }
        });

        $openModalBtn.css(options.buttonStyles);
        $openModalBtn.on('click', function () {
            $modal.show();
        });

        // backwards compatibility with version < 7
        if(options.openModalButtonTopRight){
            console.warn(LOG_LABEL, "option 'openModalButtonTopRight' deprecated; use 'openModalButtonPosition=topright' instead");
            options.openModalButtonPosition = "topright";
        }

        switch(options.openModalButtonPosition){
            // top right corner
            case "topright":
                $openModalBtn.css(
                    {
                        position: 'absolute',
                        top: '-0.5em',
                        right: '2em',
                        'z-index': 1
                    });
                $page.css('position', 'relative');
                $page.prepend($openModalBtn);
                break;
            // bottom center between next / prev buttons
            case "bottomcenter":
                var
                    buttonWidthInt = $openModalBtn.text().length * 0.7;

                $openModalBtn.css(
                    {
                        position: 'absolute',
                        'z-index': 1,
                        bottom: "2.5em",
                        left: '50%',
                        width: buttonWidthInt + "em",
                        "margin-left": ("-" + (buttonWidthInt / 2) + "em")
                    });
                $openModalBtn.insertBefore($footer);
                break;

            // inline
            case 0:
            case false:
            case null:
            case undefined:
            case "inline":

                $thisItem.append($openModalBtn);
                break;

            // after ?
            default:
                if(typeof options.openModalButtonPosition === "object"){
                    var $after = $(options.insertModalButtonAfter);
                    if(!$after.length){
                        console.error(LOGLABEL, "given DomNodeObject not found");
                        return;
                    }

                    $openModalBtn.insertAfter($after);

                }else{
                    console.error(LOG_LABEL, "unsupported button position given; expecting [[inline], topright, bottomcenter, DomNodeObject]");
                    return;
                }
                break;
        }

        this.v.destroyQueue.push(function(){
            $openModalBtn.remove();
        })
    };

    _proto.destroy = function(){
        var destroyCb = this.v.destroyQueue.pop();
        while(typeof(destroyCb) === "function"){
            destroyCb.apply(this);
            destroyCb = this.v.destroyQueue.pop();
        }
        this.v.isInit = false;
    };


    var SePlay = window.SePlay = window.SePlay || {};

    if(!SePlay.Leavesurvey){
        SePlay.Leavesurvey = new Leavesurvey();
    }

});